@@ -5,7 +5,7 @@ import pickle
 import time
 import logging
 from logging import Logger
-from typing import Type
+from typing import Type, Optional
 import matplotlib.pyplot as plt
 
 try:
@@ -13,12 +13,13 @@ try:
 except Exception:
     ImageSequenceClip = None
 
-from mohou.types import ImageBase, AngleVector, ElementDict
+from mohou.types import ImageBase, AngleVector, ElementDict, get_all_concrete_leaftypes
 from mohou.types import MultiEpisodeChunk
 from mohou.model import AutoEncoder
 from mohou.model import AutoEncoderConfig
 from mohou.model import LSTM
 from mohou.model import LSTMConfig
+from mohou.model import AutoEncoderBase
 from mohou.dataset import AutoEncoderDataset
 from mohou.dataset import AutoEncoderDatasetConfig
 from mohou.dataset import AutoRegressiveDataset
@@ -55,7 +56,9 @@ def train_autoencoder(
         use_aux_data: bool,
         model_config: AutoEncoderConfig,
         dataset_config: AutoEncoderDatasetConfig,
-        train_config: TrainConfig):
+        train_config: TrainConfig,
+        ae_type: Type[AutoEncoderBase] = AutoEncoder,
+        warm_start: bool = False):
 
     logger = create_default_logger(project_name, 'autoencoder')
 
@@ -67,9 +70,14 @@ def train_autoencoder(
         logger.info('aux data found and merged')
 
     dataset = AutoEncoderDataset.from_chunk(chunk, image_type, dataset_config)
-    tcache = TrainCache(project_name)  # type: ignore[var-annotated]
-    model = AutoEncoder(model_config)  # type: ignore
-    train(model, dataset, tcache, config=train_config)
+    if warm_start:
+        logger.info('warm start')
+        tcache = TrainCache.load(project_name, ae_type)
+        train(tcache, dataset, model=None, config=train_config)
+    else:
+        tcache = TrainCache(project_name)  # type: ignore[var-annotated]
+        model = ae_type(model_config)  # type: ignore
+        train(tcache, dataset, model=model, config=train_config)
 
 
 def train_lstm(
@@ -77,15 +85,21 @@ def train_lstm(
         embedding_rule: EmbeddingRule,
         model_config: LSTMConfig,
         dataset_config: AutoRegressiveDatasetConfig,
-        train_config: TrainConfig):
+        train_config: TrainConfig,
+        warm_start: bool = False):
 
     logger = create_default_logger(project_name, 'lstm')  # noqa
 
     chunk = MultiEpisodeChunk.load(project_name)
     dataset = AutoRegressiveDataset.from_chunk(chunk, embedding_rule, dataset_config)
-    lstm_model = LSTM(model_config)
-    tcache = TrainCache(project_name)  # type: ignore[var-annotated]
-    train(lstm_model, dataset, tcache, config=train_config)
+    if warm_start:
+        logger.info('warm start')
+        tcache = TrainCache.load(project_name, LSTM)
+        train(tcache, dataset, model=None, config=train_config)
+    else:
+        tcache = TrainCache(project_name)  # type: ignore[var-annotated]
+        model = LSTM(model_config)
+        train(tcache, dataset, model=model, config=train_config)
 
 
 def visualize_train_histories(project_name: str):
@@ -108,14 +122,15 @@ def visualize_train_histories(project_name: str):
 
 
 def visualize_image_reconstruction(
-        project_name: str, n_vis: int = 5):
+        project_name: str, n_vis: int = 5, ae_type: Type[AutoEncoderBase] = AutoEncoder):
 
     chunk = MultiEpisodeChunk.load(project_name)
     chunk_intact = chunk.get_intact_chunk()
     chunk_not_intact = chunk.get_not_intact_chunk()
 
-    tcache = TrainCache.load(project_name, AutoEncoder)
-    image_type = tcache.best_model.image_type
+    tcache = TrainCache.load(project_name, ae_type)
+    assert tcache.best_model is not None
+    image_type = tcache.best_model.image_type  # type: ignore[union-attr]
     no_aug = AutoEncoderDatasetConfig(0)  # to feed not randomized image
     dataset_intact = AutoEncoderDataset.from_chunk(chunk_intact, image_type, no_aug)
     dataset_not_intact = AutoEncoderDataset.from_chunk(chunk_not_intact, image_type, no_aug)
@@ -143,10 +158,7 @@ def visualize_image_reconstruction(
             plt.savefig(full_file_name)
 
 
-def visualize_lstm_propagation(
-        project_name: str,
-        propagator: Propagator,
-        n_prop: int):
+def visualize_lstm_propagation(project_name: str, propagator: Propagator, n_prop: int):
 
     def add_text_to_image(image: ImageBase, text: str, color: str):
         fig = plt.figure(tight_layout={'pad': 0})
@@ -159,13 +171,17 @@ def visualize_lstm_propagation(
         return canvas_to_ndarray(fig)
 
     chunk = MultiEpisodeChunk.load(project_name).get_intact_chunk()
-
     episode_data = chunk[0]
-    tcache_ae = TrainCache.load(project_name, AutoEncoder)
-    image_type = tcache_ae.best_model.image_type
+
+    image_type = None
+    for key, embedder in propagator.embed_rule.items():
+        if issubclass(key, ImageBase):
+            image_type = key
+    assert image_type is not None
+
     n_feed = 10
-    fed_avs = episode_data.filter_by_type(AngleVector)[:n_feed]
-    fed_images = episode_data.filter_by_type(image_type)[:n_feed]
+    fed_avs = episode_data.get_sequence_by_type(AngleVector)[:n_feed]
+    fed_images = episode_data.get_sequence_by_type(image_type)[:n_feed]
 
     print("start lstm propagation")
     for elem_tuple in zip(fed_avs, fed_images):
@@ -186,3 +202,23 @@ def visualize_lstm_propagation(
     assert ImageSequenceClip is not None, 'check if your moviepy is properly installed'
     clip = ImageSequenceClip(images_with_text, fps=20)
     clip.write_gif(full_file_name, fps=20)
+
+
+def auto_detect_autoencoder_type(project_name: str) -> Type[AutoEncoderBase]:
+    # TODO(HiroIshida) dirty...
+    t: Optional[Type[AutoEncoderBase]] = None
+
+    t_cand_list = get_all_concrete_leaftypes(AutoEncoderBase)
+
+    detect_count = 0
+    for t_cand in t_cand_list:
+        try:
+            TrainCache.load(project_name, t_cand)
+            t = t_cand
+            detect_count += 1
+        except Exception:
+            pass
+
+    assert detect_count == 1
+    assert t is not None  # redundant but for mypy check
+    return t
